!-----------------------------------------------------------------------------
!
! DS
!
! The subroutine DS assigns a single drainage directions to each catchment 
! cell. Two variables are also assigned  to each catchment cell:
! (1) an order numeber which is 1 for the source cells and the sum of the
! order numbers of the drained cells for the other cells (this corresponds
! to the Shreve magnitude) and (2) a logic number which is equal to the order
! number for the first cell of a link and to zero for the other cells. The
! upstream catchment area of each cell is also calculated in terms on number
! of upstream cells and stored in the variable AC_inflow.
!
!-----------------------------------------------------------------------------

subroutine ds()

use mpar
use mbbio
implicit none

integer(kind=4) i,j,i_basin,jr,l
integer(kind=4) ii,jj,ii_basin,ii_basin_ms
integer(kind=4) i_basin_chiusura
integer(kind=4) n_o_quota
real(kind=4) quota_ciijj,quota_max
real(kind=4) e(9)
real(kind=4) e0,e1,e2,emin,s_max,r_max,s_max_facet,r
integer(kind=4) sigma
real(kind=4) a1,a2
integer(kind=4) iii_1,jjj_1,iii_2,jjj_2
integer(kind=4) i_basin_cv_1,i_basin_cv_2
real(kind=4) dev_1,dev_2,sumdev,sumdev_1,sumdev_2
integer(kind=4) p_outflow_1,p_outflow_2
integer(kind=4) cell_number,link_number,p_outflow,p_inflow
integer(kind=4) cell_number_ciijj,p_outflow_ciijj
integer(kind=4) n_inflows
integer(kind=4) AC_inflow,AC_max
integer(kind=4) AC_inflow_ciijj
real(kind=4) pi,rad2

pi=4*atan(1.0E0)
rad2=sqrt(2.0E0)

! open the binary file qoi

open(16,file='qoi',access='direct',status='unknown',form='unformatted',recl=4)

! identify the outlet cell (the lowest catchment cell)

read(16,rec=N_celle+1) i_basin_chiusura

! calculate drainage directions, order numbers, logic numbers,
! and drainage areas for each catchment cell

do n_o_quota=1,N_celle
   read(16,rec=n_o_quota+1) i_basin
   jr=mod(i_basin,M)
   if (jr.ne.0) then
      j=jr
      i=(i_basin-j)/M+1
   else
      j=M
      i=i_basin/M
   end if

   l=0
   do ii=i-1,i+1
      do jj=j-1,j+1
         l=l+1
         e(l)=0.0E0
         if (ii.eq.0 .or. ii.eq.N+1) cycle
         if (jj.eq.0 .or. jj.eq.M+1) cycle
         ii_basin=(ii-1)*M+jj
         quota_ciijj= dtm_quota(ii_basin)
         if (quota_ciijj < 0) cycle
         e(l)=quota_ciijj
      end do
   end do

   e0=e(5)
   s_max=0.0E0

! triangle 021

   e1=e(2)
   e2=e(1)
   if (e1*e2.ne.0.0E0) then
      call facet(e0,e1,e2,r,s_max_facet)
      if (s_max_facet.gt.s_max) then
         r_max=r
         s_max=s_max_facet
         p_outflow_1=2
         p_outflow_2=1
         sigma=+1
      end if
   end if
!
! triangle 023
!
   e1=e(2)
   e2=e(3)
   if (e1*e2.ne.0.0E0) then
      call facet(e0,e1,e2,r,s_max_facet)
      if (s_max_facet.gt.s_max) then
         r_max=r
         s_max=s_max_facet
         p_outflow_1=2
         p_outflow_2=3
         sigma=-1
      end if
   end if

! triangle 063
!
    e1=e(6)
    e2=e(3)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=6
          p_outflow_2=3
          sigma=+1
       end if
    end if
!
! triangle 069
!
    e1=e(6)
    e2=e(9)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=6
          p_outflow_2=9
          sigma=-1
       end if
    end if
!
! triangle 089
!
    e1=e(8)
    e2=e(9)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=8
          p_outflow_2=9
          sigma=+1
       end if
    end if
!
! triangle 087
!
    e1=e(8)
    e2=e(7)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=8
          p_outflow_2=7
          sigma=-1
       end if
    end if
!
! triangle 047
!
    e1=e(4)
    e2=e(7)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=4
          p_outflow_2=7
          sigma=+1
       end if
    end if
!
! triangle 041
!
    e1=e(4)
    e2=e(1)
    if (e1*e2.ne.0.0E0) then
       call facet(e0,e1,e2,r,s_max_facet)
       if (s_max_facet.gt.s_max) then
          r_max=r
          s_max=s_max_facet
          p_outflow_1=4
          p_outflow_2=1
          sigma=-1
       end if
    end if

    if (s_max.gt.0.0E0) then

! calculate deviations dev_1 and dev_2

       if (imethod.eq.1) then ! D8-LAD method
          dev_1=r_max ! delta_x=delta_y
          dev_2=pi/4-r_max ! delta_x=delta_y
       else if (imethod.eq.2) then ! D8-LTD method
          dev_1=delta_x*sin(r_max) ! delta_x=delta_y
          dev_2=delta_x*rad2*sin(pi/4-r_max) ! delta_x=delta_y
       else
          write(6,*) 'unespected imethod!'
          stop
       end if

       if (sigma.eq.+1) then
          dev_2=-dev_2
       else ! if (sigma.eq.-1) then
          dev_1=-dev_1
       end if

       n_inflows=0
       cell_number=0
       AC_inflow=0
       AC_max=0
       quota_max=-1000.0
       do ii=i-1,i+1
          do jj=j-1,j+1
             if (ii.eq.0 .or. ii.eq.N+1) cycle
             if (jj.eq.0 .or. jj.eq.M+1) cycle
             ii_basin=(ii-1)*M+jj
             if (dtm_index_pr(ii_basin) == 0) cycle
             cell_number_ciijj=dtm_cell_number(ii_basin)
             if (cell_number_ciijj.eq.0) cycle ! cella non processata
             p_outflow_ciijj=dtm_p_outflow(ii_basin)

             p_inflow=3*(ii-i)+(jj-j)+5
             if (p_inflow+p_outflow_ciijj.eq.10) then
                n_inflows=n_inflows+1
                quota_ciijj=dtm_quota(ii_basin)
                AC_inflow_ciijj=dtm_AC_inflow(ii_basin)
                cell_number=cell_number+cell_number_ciijj
                AC_inflow=AC_inflow+AC_inflow_ciijj+1
                if (AC_inflow_ciijj.gt.AC_max) then
                   AC_max=AC_inflow_ciijj
                   quota_max=quota_ciijj
                   ii_basin_ms=ii_basin
                else if (AC_inflow_ciijj.eq.AC_max) then
                   AC_max=AC_inflow_ciijj ! non serve
                   if (quota_ciijj.gt.quota_max) then
                      quota_max=quota_ciijj
                      ii_basin_ms=ii_basin
                   end if
                end if
             end if
          end do
       end do
       if (n_inflows.ge.1) then
          sumdev=dtm_sumdev(ii_basin_ms)
       else ! if (n_inflows.eq.0) then
          sumdev=0.0E0
          cell_number=1
       end if
       sumdev_1=dev_1+lambda*sumdev
       sumdev_2=dev_2+lambda*sumdev

! the function p_outflow is the coordinate z of the plane ax+by+c=z,
! with a=3, b=1 and c=5. the function (i_basin_cv-i_basin) is the coordinate
! z of the plane ax+by+c=z, with a=M, b=1 e c=0.

       iii_1=nint((p_outflow_1-5)/3.)
       jjj_1=p_outflow_1-5-3*iii_1
       i_basin_cv_1=i_basin+(M*iii_1+jjj_1)
       e1=dtm_quota(i_basin_cv_1)

       iii_2=nint((p_outflow_2-5)/3.)
       jjj_2=p_outflow_2-5-3*iii_2
       i_basin_cv_2=i_basin+(M*iii_2+jjj_2)
       e2=dtm_quota(i_basin_cv_2)

       a1=abs(sumdev_1)
       a2=abs(sumdev_2)
       if (a1.le.a2.and.(e0-e1).gt.0.0) then
          p_outflow=p_outflow_1
          sumdev=sumdev_1
       else if (a1.gt.a2.or.(e0-e2).gt.0.0) then
          p_outflow=p_outflow_2
          sumdev=sumdev_2
       else ! non serve
          write(6,*) 's_max > 0, unexpected case!'
          write(6,*) 'i = ',i
          write(6,*) 'j = ',j
          stop
       end if

    else if (s_max.eq.0.0E0) then

       write(6,*) 's_max = 0 at i_basin = ',i_basin
       if (i_basin.eq.i_basin_chiusura) then
          n_inflows=0
          cell_number=0
          AC_inflow=0
          AC_max=0
          quota_max=-1000.0
          do ii=i-1,i+1
             do jj=j-1,j+1
                if (ii.eq.0 .or. ii.eq.N+1) cycle
                if (jj.eq.0 .or. jj.eq.M+1) cycle
                ii_basin=(ii-1)*M+jj
                if (dtm_index_pr(ii_basin) == 0) cycle
                cell_number_ciijj=dtm_cell_number(ii_basin)

                if (cell_number_ciijj.eq.0) cycle ! cella non processata
                p_outflow_ciijj=dtm_p_outflow(ii_basin)
                p_inflow=3*(ii-i)+(jj-j)+5
                if (p_inflow+p_outflow_ciijj.eq.10) then
                   n_inflows=n_inflows+1
                   quota_ciijj=dtm_quota(ii_basin)
                   AC_inflow_ciijj=dtm_AC_inflow(ii_basin)
                   cell_number=cell_number+cell_number_ciijj
                   AC_inflow=AC_inflow+ AC_inflow_ciijj+1
                   if (AC_inflow_ciijj.gt.AC_max) then
                      AC_max=AC_inflow_ciijj
                      quota_max=quota_ciijj
                      ii_basin_ms=ii_basin
                   else if (AC_inflow_ciijj.eq.AC_max) then
                      AC_max=AC_inflow_ciijj ! non serve
                      if (quota_ciijj.gt.quota_max) then
                         quota_max=quota_ciijj
                         ii_basin_ms=ii_basin
                      end if
                   end if
                end if
             end do
          end do
          if (n_inflows.ge.1) then
             sumdev=dtm_sumdev(ii_basin_ms)
          else ! if (n_inflows.eq.0) then
             sumdev=0.0E0
             cell_number=1
          end if
          p_outflow=0 ! conventionally
          sumdev=lambda*sumdev ! conventionally
          if (AC_inflow.ne.(N_celle-1)) then
             write(6,*) 'inexact catchment area!'
          end if
       else ! if (i_basin.ne.i_basin_chiusura) then
          n_inflows=0
          cell_number=0
          AC_inflow=0
          AC_max=0
          quota_max=-1000.0
          do ii=i-1,i+1
             do jj=j-1,j+1
                if (ii.eq.0 .or. ii.eq.N+1) cycle
                if (jj.eq.0 .or. jj.eq.M+1) cycle
                ii_basin=(ii-1)*M+jj
                if (dtm_index_pr(ii_basin)== 0) cycle
                cell_number_ciijj=dtm_cell_number(ii_basin)
                if (cell_number_ciijj.eq.0) cycle ! cella non processata
                p_outflow_ciijj=dtm_p_outflow(ii_basin)
                p_inflow=3*(ii-i)+(jj-j)+5
                if (p_inflow+p_outflow_ciijj.eq.10) then
                   n_inflows=n_inflows+1
                   quota_ciijj=dtm_quota(ii_basin)
                   AC_inflow_ciijj=dtm_AC_inflow(ii_basin)
                   cell_number=cell_number+cell_number_ciijj
                   AC_inflow=AC_inflow+AC_inflow_ciijj+1
                   if (AC_inflow_ciijj.gt.AC_max) then
                      AC_max=AC_inflow_ciijj
                      quota_max=quota_ciijj
                      ii_basin_ms=ii_basin
                   else if (AC_inflow_ciijj.eq.AC_max) then
                      AC_max=AC_inflow_ciijj ! non serve
                      if (quota_ciijj.gt.quota_max) then
                         quota_max=quota_ciijj
                         ii_basin_ms=ii_basin
                      end if
                   end if
                end if
             end do
          end do
          if (n_inflows.ge.1) then
             sumdev=dtm_sumdev(ii_basin_ms)
          else ! if (n_inflows.eq.0) then
             sumdev=0.0E0
             cell_number=1
          end if
          emin=9000.0
          p_outflow=0
          do l=1,9
             if (e(l).ne.0.0.and.l.ne.5.and.e(l).lt.emin) then
                emin=e(l)
                p_outflow=l
             end if
          end do
          if (p_outflow.eq.0) then
             write(6,*) 's_max = 0, unexpected case!'
             write(6,*) 'i = ',i
             write(6,*) 'j = ',j
             stop
          end if
          sumdev=lambda*sumdev ! conventionally
       end if

    else ! if (s_max.lt.0.0E0) then

       write(6,*) 's_max < 0'
       write(6,*) 'i = ',i
       write(6,*) 'j = ',j
       stop

    end if

    if (n_inflows.eq.0.or.n_inflows.gt.1) then
       link_number=cell_number
    else ! if (n_inflows.eq.1) then
       link_number=0
    end if

    call set_p_outflow(i_basin,p_outflow)
    call set_cell_number(i_basin,cell_number)
    call set_link_number(i_basin,link_number)
    call set_AC_inflow(i_basin,AC_inflow)
    call set_sumdev(i_basin,sumdev)
end do

close(16)

return
end subroutine ds
