!-----------------------------------------------------------------------------
!
! HC
!
! The subroutine HC (Hillslopes and Channels) determine the hillslope an
! channel cells in the catchment. It also assign to each DEM cell the
! hydraulic geometry parameters.
! Note that the upstream drainage area can either be expressed in terms
! of square kilometers (variables referring to A, upstream drainage area) 
! or in terms of catchment cells (variables referring to AC, upstream 
! drainage cells).
!
!-----------------------------------------------------------------------------

subroutine hc()

use mpar
use mbbio
implicit none

integer(kind=4) nuc,hcID_ciijj,hcID
integer(kind=4) i,j,i_basin, jr
integer(kind=4) ii,jj,iii,jjj,ii_basin,i_basin_cv
integer(kind=4) i_basin_io
integer(kind=4) i_basin_chiusura
integer(kind=4) p_outflow,link_number,link_number_cv,p_inflow
integer(kind=4) p_outflow_ciijj
real(kind=4) S_mean
real(kind=4) DL,DN,ASk
integer(kind=4) n_links,l_link
integer(kind=4) AC_inflow
real(kind=4) A_cell,A_basin,A_outflow
real(kind=4) link_slope
real(kind=4) Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn,Pgwn,d1gwn

integer, external :: ChannelInitiation

A_cell=delta_x*delta_y
A_basin=N_celle*A_cell

! calculate the mean local slope of the catchment S_mean

S_mean=0.0E0
do i=1,N
   do j=1,M
      i_basin=(i-1)*M+j
      if (dtm_index_pr(i_basin) == 0) cycle
      S_mean=S_mean+dtm_local_slope(i_basin) ! 21
   end do
end do
S_mean=S_mean/N_celle
if (S_mean.eq.0.0E0) then
   write(6,*) 'S_mean=0!'
   stop
else
   write(6,*) 'S_mean=',S_mean
end if

! calculate the normalized divergence DN=DL/(-S_mean)

do i=1,N
   do j=1,M
      i_basin=(i-1)*M+j
      if (dtm_index_pr(i_basin) == 0) cycle
      DL=dtm_DN(i_basin)
      DN=DL/(-S_mean)
      call set_DN(i_basin,DN)
   end do
end do

! open the binary file qoi to determine the outlet cell

open(16,file='qoi',access='direct',status='unknown',form='unformatted',recl=4)
read(16,rec=N_celle+1) i_basin_chiusura
close(16)

! open the binary file ioi

open(17,file='ioi',access='direct',status='unknown',form='unformatted',recl=4)

! determine the number of links in the catchment

read(17,rec=1) n_links

! determine the hydraulic geometry for the outlet of each DEM cell by
! processing all the links from that of order 1 to the second last one
! (the oulet link is processed separately)

do l_link=1,n_links-1

! process the uppermost cell of the link

   read(17,rec=l_link+1) i_basin_io
   i_basin=i_basin_io
   p_outflow=dtm_p_outflow(i_basin)     ! 14
   link_number=dtm_link_number(i_basin) ! 16
   AC_inflow=dtm_AC_inflow(i_basin)     ! 17
   link_slope=dtm_link_slope(i_basin)   ! 22
   Wsn=dtm_Wsn(i_basin)                 ! 25
   b1sn=dtm_b1sn(i_basin)               ! 26
   Psn=dtm_Psn(i_basin)                 ! 27
   d1sn=dtm_d1sn(i_basin)               ! 28
   Wgwn=dtm_Wgwn(i_basin)               ! 29
   b1gwn=dtm_b1gwn(i_basin)             ! 30
   Pgwn=dtm_Pgwn(i_basin)               ! 31
   d1gwn=dtm_d1gwn(i_basin)             ! 32
   ASk=dtm_ASk(i_basin)                 ! 33
   DN=dtm_DN(i_basin)                   ! 34

! calculate the upstream drainage area for the outlet of each cell

   A_outflow=AC_inflow*A_cell+A_cell

! p_outflow is treated here as the coordinate z of the palne ax+by+c=z
! where a=3, b=1 and c=5; i_basin_cv-i_basin is treated here as the
! coordinate z of the plane ax+by+c=z, where a=M, b=1 and c=0

   iii=nint((p_outflow-5)/3.)
   jjj=p_outflow-5-3*iii
   i_basin_cv=i_basin+(M*iii+jjj)
   link_number_cv=dtm_link_number(i_basin_cv)

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

   nuc=0 ! link_number=1
   if (link_number.gt.1) then
      jr=mod(i_basin,M)
      if (jr.ne.0) then
         j=jr
         i=(i_basin-j)/M+1
      else
         j=M
         i=i_basin/M
      end if
      do ii=i-1,i+1
         do jj=j-1,j+1
            if (ii.eq.0 .or. ii.eq.N+1) cycle
            if (jj.eq.0 .or. jj.eq.M+1) cycle
            ii_basin=(ii-1)*M+jj
            if (dtm_index_pr(ii_basin) == 0) cycle
            p_outflow_ciijj=dtm_p_outflow(ii_basin)

            p_inflow=3*(ii-i)+(jj-j)+5
            if (p_inflow+p_outflow_ciijj.eq.10) then
               hcID_ciijj=dtm_hcID(ii_basin)
               if (hcID_ciijj.eq.1) nuc=max(nuc,hcID_ciijj)
            end if
         end do
      end do
   end if

   if (nuc.eq.0) then
      hcID=ChannelInitiation(A_outflow,ASk,DN)
   else
      hcID=1
   end if

   call HydraulicGeometry(hcID,A_outflow,A_basin, &
   Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )

   call set_link_slope(i_basin,link_slope) ! 22
   call set_Wsn(i_basin,Wsn)               ! 25
   call set_b1sn(i_basin,b1sn)             ! 26
   call set_Psn(i_basin,Psn)               ! 27
   call set_d1sn(i_basin,d1sn)             ! 28
   call set_Wgwn(i_basin,Wgwn)             ! 29
   call set_b1gwn(i_basin,b1gwn)           ! 30
   call set_Pgwn(i_basin,Pgwn)             ! 31
   call set_d1gwn(i_basin,d1gwn)           ! 32
   call set_hcID(i_basin,hcID)             ! 35

! process the subsequent cells of the link

   nuc=hcID
   link_number=link_number_cv
   do while(link_number.eq.0)
      i_basin=i_basin_cv
      p_outflow=dtm_p_outflow(i_basin)     ! 14
      link_number=dtm_link_number(i_basin) ! 16
      AC_inflow=dtm_AC_inflow(i_basin)     ! 17
      Wsn=dtm_Wsn(i_basin)                 ! 25
      b1sn=dtm_b1sn(i_basin)               ! 26
      Psn=dtm_Psn(i_basin)                 ! 27
      d1sn=dtm_d1sn(i_basin)               ! 28
      Wgwn=dtm_Wgwn(i_basin)               ! 29
      b1gwn=dtm_b1gwn(i_basin)             ! 30
      Pgwn=dtm_Pgwn(i_basin)               ! 31
      d1gwn=dtm_d1gwn(i_basin)             ! 32
      ASk=dtm_ASk(i_basin)                 ! 33
      DN=dtm_DN(i_basin)                   ! 34 

! calculate the upstream drainage area for the outlet of each cell

      A_outflow=AC_inflow*A_cell+A_cell

      iii=nint((p_outflow-5)/3.)
      jjj=p_outflow-5-3*iii
      i_basin_cv=i_basin+(M*iii+jjj)
      link_number_cv=dtm_link_number(i_basin_cv)

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

      if (nuc.eq.0) then
         hcID=ChannelInitiation(A_outflow,ASk,DN)
      else
         hcID=1
      end if

      call HydraulicGeometry(hcID,A_outflow,A_basin, & 
      Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )

      call set_link_slope(i_basin,link_slope) ! 22
      call set_Wsn(i_basin,Wsn)               ! 25
      call set_b1sn(i_basin,b1sn)             ! 26
      call set_Psn(i_basin,Psn)               ! 27
      call set_d1sn(i_basin,d1sn)             ! 28
      call set_Wgwn(i_basin,Wgwn)             ! 29
      call set_b1gwn(i_basin,b1gwn)           ! 30
      call set_Pgwn(i_basin,Pgwn)             ! 31
      call set_d1gwn(i_basin,d1gwn)           ! 32
      call set_hcID(i_basin,hcID)             ! 35
      nuc=hcID
      link_number=link_number_cv
   end do
end do

! process the outlet link of the catchment
! deternine the uppermost cell

read(17,rec=n_links+1) i_basin_io
i_basin=i_basin_io
p_outflow=dtm_p_outflow(i_basin)
link_number=dtm_link_number(i_basin)
AC_inflow=dtm_AC_inflow(i_basin)
link_slope=dtm_link_slope(i_basin)
Wsn=dtm_Wsn(i_basin)
b1sn=dtm_b1sn(i_basin)
Psn=dtm_Psn(i_basin)
d1sn=dtm_d1sn(i_basin)
Wgwn=dtm_Wgwn(i_basin)
b1gwn=dtm_b1gwn(i_basin)
Pgwn=dtm_Pgwn(i_basin)
d1gwn=dtm_d1gwn(i_basin)
ASk=dtm_ASk(i_basin)
DN=dtm_DN(i_basin)

if (i_basin.eq.i_basin_chiusura) then
   write(6,*) 'i_basin_chiusura = ',i_basin_chiusura

! calculate the upstream drainage area for the outlet of each cell

   A_outflow=AC_inflow*A_cell+A_cell

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

   nuc=0 ! link_number=1
   if (link_number.gt.1) then
      jr=mod(i_basin,M)
      if (jr.ne.0) then
         j=jr
         i=(i_basin-j)/M+1
      else
         j=M
         i=i_basin/M
      end if
      do ii=i-1,i+1
         do jj=j-1,j+1
            if (ii.eq.0 .or. ii.eq.N+1) cycle
            if (jj.eq.0 .or. jj.eq.M+1) cycle
            ii_basin=(ii-1)*M+jj
            if (dtm_index_pr(ii_basin) == 0) cycle
            p_outflow_ciijj=dtm_p_outflow(ii_basin)
            p_inflow=3*(ii-i)+(jj-j)+5
            if (p_inflow+p_outflow_ciijj.eq.10) then
               hcID_ciijj=dtm_hcID(ii_basin)
               if (hcID_ciijj.eq.1) nuc=max(nuc,hcID_ciijj)
            end if
         end do
      end do
   end if

   if (nuc.eq.0) then
      hcID=ChannelInitiation(A_outflow,ASk,DN)
   else
      hcID=1
   end if

   call HydraulicGeometry(hcID,A_outflow,A_basin, &
   Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )

   call set_link_slope(i_basin,link_slope) ! 22
   call set_Wsn(i_basin,Wsn)               ! 25
   call set_b1sn(i_basin,b1sn)             ! 26
   call set_Psn(i_basin,Psn)               ! 27
   call set_d1sn(i_basin,d1sn)             ! 28
   call set_Wgwn(i_basin,Wgwn)             ! 29
   call set_b1gwn(i_basin,b1gwn)           ! 30
   call set_Pgwn(i_basin,Pgwn)             ! 31
   call set_d1gwn(i_basin,d1gwn)           ! 32
   call set_hcID(i_basin,hcID)             ! 35

else

! calculate the upstream drainage area for the outlet of each cell

   A_outflow=AC_inflow*A_cell+A_cell

   iii=nint((p_outflow-5)/3.)
   jjj=p_outflow-5-3*iii
   i_basin_cv=i_basin+(M*iii+jjj)
   link_number_cv=dtm_link_number(i_basin_cv)

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

   nuc=0 ! link_number=1
   if (link_number.gt.1) then
      jr=mod(i_basin,M)
      if (jr.ne.0) then
         j=jr
         i=(i_basin-j)/M+1
      else
         j=M
         i=i_basin/M
      end if
      do ii=i-1,i+1
         do jj=j-1,j+1
            if (ii.eq.0 .or. ii.eq.N+1) cycle
            if (jj.eq.0 .or. jj.eq.M+1) cycle
            ii_basin=(ii-1)*M+jj
            if (dtm_index_pr(ii_basin) == 0) cycle
            p_outflow_ciijj=dtm_p_outflow(ii_basin)
            p_inflow=3*(ii-i)+(jj-j)+5
            if (p_inflow+p_outflow_ciijj.eq.10) then
               hcID_ciijj=dtm_hcID(ii_basin)
               if (hcID_ciijj.eq.1) nuc=max(nuc,hcID_ciijj)
            end if
         end do
      end do
   end if

   if (nuc.eq.0) then
      hcID=ChannelInitiation(A_outflow,ASk,DN)
   else
      hcID=1
   end if

   call HydraulicGeometry(hcID,A_outflow,A_basin, &
   Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )

   call set_link_slope(i_basin,link_slope) ! 22
   call set_Wsn(i_basin,Wsn)               ! 25
   call set_b1sn(i_basin,b1sn)             ! 26
   call set_Psn(i_basin,Psn)               ! 27
   call set_d1sn(i_basin,d1sn)             ! 28
   call set_Wgwn(i_basin,Wgwn)             ! 29
   call set_b1gwn(i_basin,b1gwn)           ! 30
   call set_Pgwn(i_basin,Pgwn)             ! 31
   call set_d1gwn(i_basin,d1gwn)           ! 32
   call set_hcID(i_basin,hcID)             ! 35

! subsequent cells of the outlet link (the outlet cell should have logic
! numeber link_number=0 except it is the first cell of the link)

   nuc=hcID
   link_number=link_number_cv
   do while(link_number.eq.0)

      i_basin=i_basin_cv
      p_outflow=dtm_p_outflow(i_basin)
      link_number=dtm_link_number(i_basin)
      AC_inflow=dtm_AC_inflow(i_basin)
      Wsn=dtm_Wsn(i_basin)
      b1sn=dtm_b1sn(i_basin)
      Psn=dtm_Psn(i_basin)
      d1sn=dtm_d1sn(i_basin)
      Wgwn=dtm_Wgwn(i_basin)
      b1gwn=dtm_b1gwn(i_basin)
      Pgwn=dtm_Pgwn(i_basin)
      d1gwn=dtm_d1gwn(i_basin)
      ASk=dtm_ASk(i_basin)
      DN=dtm_DN(i_basin)

      if (i_basin.eq.i_basin_chiusura) then
         write(6,*)'cella di chiusura i_basin_chiusura=',i_basin_chiusura

! calculate the upstream drainage area for the outlet of each cell

         A_outflow=AC_inflow*A_cell+A_cell

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

         if (nuc.eq.0) then
            hcID=ChannelInitiation(A_outflow,ASk,DN)
         else
            hcID=1
         end if

         call HydraulicGeometry(hcID,A_outflow,A_basin, &
         Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )

         call set_link_slope(i_basin,link_slope) ! 25
         call set_b1sn(i_basin,b1sn)             ! 26
         call set_Psn(i_basin,Psn)               ! 27
         call set_d1sn(i_basin,d1sn)             ! 28
         call set_Wgwn(i_basin,Wgwn)             ! 29
         call set_b1gwn(i_basin,b1gwn)           ! 30
         call set_Pgwn(i_basin,Pgwn)             ! 31
         call set_d1gwn(i_basin,d1gwn)           ! 32
         call set_hcID(i_basin,hcID)             ! 35

         link_number=1 ! i.e. exit from the do while cycle

      else

! calculate the upstream drainage area for the outlet of each cell

         A_outflow=AC_inflow*A_cell+A_cell

         iii=nint((p_outflow-5)/3.)
         jjj=p_outflow-5-3*iii
         i_basin_cv=i_basin+(M*iii+jjj)
         link_number_cv=dtm_link_number(i_basin_cv)

! determine hillslope and channel cells
! calculate the hydraulic geometry parameters of each cells

         if (nuc.eq.0) then
            hcID=ChannelInitiation(A_outflow,ASk,DN)
         else
            hcID=1
         end if

         call HydraulicGeometry(hcID,A_outflow,A_basin, &
         Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn,Pgwn,d1gwn)

         call set_link_slope(i_basin,link_slope) ! 22
         call set_Wsn(i_basin,Wsn)               ! 25
         call set_b1sn(i_basin,b1sn)             ! 26
         call set_Psn(i_basin,Psn)               ! 27
         call set_d1sn(i_basin,d1sn)             ! 28
         call set_Wgwn(i_basin,Wgwn)             ! 29
         call set_b1gwn(i_basin,b1gwn)           ! 30
         call set_Pgwn(i_basin,Pgwn)             ! 31
         call set_d1gwn(i_basin,d1gwn)           ! 32
         call set_hcID(i_basin,hcID)             ! 35
         nuc=hcID
         link_number=link_number_cv ! i.e. go for another cycle

      end if
   end do
end if

! close the file ioi

close(17)

return
end subroutine hc


function ChannelInitiation(A_outflow,ASk,DN)
use mpar
implicit none
integer :: ChannelInitiation
real(kind=4),intent(in) :: A_outflow,ASk,DN
select case (nchc)
case (1)
   if (A_outflow.lt.A_threshold) then
      ChannelInitiation=0
     else
      ChannelInitiation=1
   end if
case (2)
   if (ASk.lt.ASk_threshold) then
      ChannelInitiation=0
   else
      ChannelInitiation=1
   end if
case (3)
   if (DN.ge.DN_threshold) then
      ChannelInitiation=0
   else
      ChannelInitiation=1
   end if
case default
   write(6,*) 'nchc fuori campo!'
   stop
end select
end function ChannelInitiation

subroutine HydraulicGeometry(hcID,A_outflow,A_basin, &
Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn, Pgwn, d1gwn )
use mpar
implicit none
integer,intent(in) :: hcID
real,intent(inout) :: Wsn,b1sn,Psn,d1sn,Wgwn,b1gwn,Pgwn, d1gwn
real,intent(in) :: A_outflow, A_basin
real :: RA

RA=A_outflow/A_basin

if (hcID.eq.0) then ! hillslope
   if (Wsn.eq.0.0E0) Wsn=W_R0_sn_h*RA**(b2_sn_h-b1_sn_h)
   if (b1sn.eq.0.0E0) b1sn=b1_sn_h
   if (Psn.eq.0.0E0) Psn=P_R0_sn_h*RA**(d2_sn_h-d1_sn_h)
   if (d1sn.eq.0.0E0) d1sn=d1_sn_h
   if (Wgwn.eq.0.0E0) Wgwn=W_R0_gwn_h*RA**(b2_gwn_h-b1_gwn_h)
   if (b1gwn.eq.0.0E0) b1gwn=b1_gwn_h
   if (Pgwn.eq.0.0E0) Pgwn=P_R0_gwn_h*RA**(d2_gwn_h-d1_gwn_h)
   if (d1gwn.eq.0.0E0) d1gwn=d1_gwn_h
else
   if (Wsn.eq.0.0E0) Wsn=W_R0_sn_c*RA**(b2_sn_c-b1_sn_c)
   if (b1sn.eq.0.0E0) b1sn=b1_sn_c
   if (Psn.eq.0.0E0) Psn=P_R0_sn_c*RA**(d2_sn_c-d1_sn_c)
   if (d1sn.eq.0.0E0) d1sn=d1_sn_c
   if (Wgwn.eq.0.0E0) Wgwn=W_R0_gwn_c*RA**(b2_gwn_c-b1_gwn_c)
   if (b1gwn.eq.0.0E0) b1gwn=b1_gwn_c
   if (Pgwn.eq.0.0E0) Pgwn=P_R0_gwn_c*RA**(d2_gwn_c-d1_gwn_c)
   if (d1gwn.eq.0.0E0) d1gwn=d1_gwn_c
end if
end subroutine HydraulicGeometry
