!-----------------------------------------------------------------------------
!
! LPF
!
! The subroutine LPF (Local Physiographic Features) calculates the
! physiographic features of the catchment at each DEM cell.
! Note that the upstream drainage area can either be expressed in terms
! of square kilometers (variables referring to A, upstream drainage area) 
! or in terms of catchment cells (variables referring to AC, upstream 
! drainage cells).
!
!-----------------------------------------------------------------------------

subroutine lpf()

use mpar
use mbbio
implicit none

integer(kind=4) i,j, i_basin,i_basin_cm,i_basin_cv,i_basin_io
integer(kind=4) ii,jj, ii_basin,iii,jjj, jr
integer(kind=4) i_basin_sorgente_l_link,i_basin_chiusura
integer(kind=4) l_link,n_links
integer(kind=4) p_outflow,link_number,link_number_cv,p_inflow
integer(kind=4) p_outflow_ciijj,p_outflow_cm
integer(kind=4) AC_inflow,AC_inflow_ciijj,AC_max
real(kind=4) delta_xy,A_cell,A_outflow
real(kind=4) quota,quota_cm,quota_cv,quota_max,quota_ciijj
real(kind=4) d1,d2,z1,z2,z3
real(kind=4) epl,local_slope
real(kind=4) DL,ASk
real(kind=4) link_slope,link_slope_num, link_slope_den

delta_xy=sqrt(2.0)*delta_x ! assumendo delta_x=delta_y
A_cell=delta_x*delta_y

! open the binary file qoi to determine the outlet cell 

open(16,file='qoi',access='direct',status='unknown',form='unformatted',recl=4)
read(16,rec=N_celle+1) i_basin_chiusura
close(16)

! open the binary file ioi

open(17,file='ioi',access='direct',status='unknown',form='unformatted',recl=4)

! determine the number of links in the catchment

read(17,rec=1) n_links

! process all the links, from that of order 1 to the second last one
! (the oulet link is c processed separately)

do l_link=1,n_links-1

! first cell of the link

   read(17,rec=l_link+1) i_basin_io
   i_basin_sorgente_l_link=i_basin_io
   i_basin=i_basin_io
   quota=           dtm_quota(i_basin)
   p_outflow=     dtm_p_outflow(i_basin)
   link_number=        dtm_link_number(i_basin)
   AC_inflow= dtm_AC_inflow(i_basin)
   A_outflow=AC_inflow*A_cell+A_cell

! p_outflow is treated here as the coordinate z of the palne ax+by+c=z, 
! where a=3, b=1 and c=5; i_basin_cv-i_basin is treated here as the
! coordinate z of the plane ax+by+c=z, where a=M, b=1 and c=0

  iii=nint((p_outflow-5)/3.)
  jjj=p_outflow-5-3*iii
  i_basin_cv=i_basin+(M*iii+jjj)
  quota_cv=    dtm_quota   (i_basin_cv)
  link_number_cv= dtm_link_number(i_basin_cv)

! calculate: (1) the elemenatal length of the channel within each DEM cell 
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

  if (link_number.eq.1) then
     if (mod(p_outflow,2).eq.1) then
        d2=delta_xy
     else ! if (mod(p_outflow,2).eq.0) then
        d2=delta_x
     end if
     z1=quota ! phantom
     z2=quota
     z3=quota_cv
     d1=d2 ! phantom
     epl=d2
     local_slope=(z2-z3)/d2
     DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
  else ! if (link_number.gt.1) then
     jr=mod(i_basin,M)
     if (jr.ne.0) then
        j=jr
        i=(i_basin-j)/M+1
     else
        j=M
        i=i_basin/M
     end if
     AC_max=0
     quota_max=-1000.0
     do ii=i-1,i+1
        do jj=j-1,j+1
           if (ii.eq.0 .or. ii.eq.N+1) cycle
           if (jj.eq.0 .or. jj.eq.M+1) cycle
           ii_basin=(ii-1)*M+jj
           if (dtm_index_pr(ii_basin) == 0) cycle
           p_outflow_ciijj= dtm_p_outflow(ii_basin)
           p_inflow=3*(ii-i)+(jj-j)+5
           if (p_inflow+p_outflow_ciijj.eq.10) then
              quota_ciijj=dtm_quota(ii_basin)
              AC_inflow_ciijj= dtm_AC_inflow(ii_basin)
              if (AC_inflow_ciijj.gt.AC_max) then
                 AC_max=AC_inflow_ciijj
                 quota_max=quota_ciijj
                 z1=quota_ciijj
                 if (mod(p_outflow_ciijj,2).eq.1) then
                    d1=delta_xy
                 else ! if (mod(p_outflow_ciijj,2).eq.0) then
                    d1=delta_x
                 end if
              else if (AC_inflow_ciijj.eq.AC_max) then
                 AC_max=AC_inflow_ciijj ! non serve
                 if (quota_ciijj.gt.quota_max) then
                    quota_max=quota_ciijj
                    z1=quota_ciijj
                    if (mod(p_outflow_ciijj,2).eq.1) then
                       d1=delta_xy
                    else ! if (mod(p_outflow_ciijj,2).eq.0) then
                       d1=delta_x
                    end if
                 end if
              end if
           end if
        end do
     end do
     z2=quota
     z3=quota_cv
     if (mod(p_outflow,2).eq.1) then
        d2=delta_xy
     else ! if (mod(p_outflow,2).eq.0) then
        d2=delta_x
     end if
     epl=d2
     local_slope=(z2-z3)/d2
     DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
  end if
  ASk=A_outflow*local_slope**kas

  if (local_slope.le.0.0E0) then
     write(6,*)'i_basin=',i_basin
     write(6,*)'local_slope=<0'
     local_slope=local_slope_t
  end if

  call set_epl(i_basin,epl)
  call set_local_slope(i_basin,local_slope)
  call set_ASk(i_basin,ASk)
  call set_DN(i_basin,DL)

  link_slope_num=local_slope*epl
  link_slope_den=epl

! subsequent cells of the link 

  link_number=link_number_cv
  do while(link_number.eq.0)
     i_basin_cm=i_basin
     quota_cm=dtm_quota(i_basin_cm)
     p_outflow_cm=dtm_p_outflow(i_basin_cm)

     i_basin=i_basin_cv
     quota=dtm_quota(i_basin)
     p_outflow=dtm_p_outflow(i_basin)
     link_number=dtm_link_number(i_basin)
     AC_inflow=dtm_AC_inflow(i_basin)
     A_outflow=AC_inflow*A_cell+A_cell

     iii=nint((p_outflow-5)/3.)
     jjj=p_outflow-5-3*iii
     i_basin_cv=i_basin+(M*iii+jjj)
     quota_cv=dtm_quota(i_basin_cv)
     link_number_cv=dtm_link_number(i_basin_cv)

! calculate: (1) the elemenatal length of the channel within each DEM cell 
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

     z1=quota_cm
     z2=quota
     z3=quota_cv
     if (mod(p_outflow_cm,2).eq.1) then
        d1=delta_xy
     else ! if (mod(p_outflow_cm,2).eq.0) then
        d1=delta_x
     end if
     if (mod(p_outflow,2).eq.1) then
        d2=delta_xy
     else ! if (mod(p_outflow,2).eq.0) then
        d2=delta_x
     end if
     epl=d2
     local_slope=(z2-z3)/d2
     DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
     ASk=A_outflow*local_slope**kas

     if (local_slope.le.0.0E0) then
        write(6,*)'i_basin=',i_basin
        write(6,*)'local_slope=<0'
        local_slope=local_slope_t
     end if

     call set_epl(i_basin,epl)
     call set_local_slope(i_basin,local_slope)
     call set_ASk(i_basin,ASk)
     call set_DN(i_basin,DL)

     link_slope_num=link_slope_num+ local_slope*epl
     link_slope_den=link_slope_den+epl

     link_number=link_number_cv
  end do

  link_slope=link_slope_num/link_slope_den
  if (link_slope.le.0.0E0) then
     write(6,*)'link_slope=',link_slope
     write(6,*)'i_basin=',i_basin
     stop "stop in lpf"
  end if

! write link_slope in the uppermost cell of the link (this will 
! reported in the subsequent cell of the link by the subroutine HC)

  i_basin=i_basin_sorgente_l_link
  call set_link_slope(i_basin,link_slope)

end do

! outlet link of the catchment

! uppermost cell of the outlet link of the catchment

read(17,rec=n_links+1) i_basin_io
i_basin_sorgente_l_link=i_basin_io
i_basin=i_basin_io

quota=dtm_quota(i_basin)
p_outflow=dtm_p_outflow(i_basin)
link_number=dtm_link_number(i_basin)
AC_inflow=dtm_AC_inflow(i_basin)
A_outflow=AC_inflow*A_cell+A_cell

if (i_basin.eq.i_basin_chiusura) then
   write(6,*) 'i_basin_chiusura = ',i_basin_chiusura

! calculate: (1) the elemenatal length of the channel within each DEM cell 
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

   if (link_number.eq.1) then
      write(6,*) 'catchment with one cell!'
      write(6,*) 'undetermined slope and convexity!'
      stop "stop in lpf"
   else ! if (link_number.gt.1) then
      jr=mod(i_basin,M)
      if (jr.ne.0) then
         j=jr
         i=(i_basin-j)/M+1
      else
         j=M
         i=i_basin/M
      end if
      AC_max=0
      quota_max=-1000.0
      do ii=i-1,i+1
         do jj=j-1,j+1
            if (ii.eq.0 .or. ii.eq.N+1) cycle
            if (jj.eq.0 .or. jj.eq.M+1) cycle
            ii_basin=(ii-1)*M+jj
            if (dtm_index_pr(ii_basin) == 0) cycle
            p_outflow_ciijj= dtm_p_outflow(ii_basin)

            p_inflow=3*(ii-i)+(jj-j)+5
            if (p_inflow+p_outflow_ciijj.eq.10) then
               quota_ciijj=dtm_quota(ii_basin)
               AC_inflow_ciijj= dtm_AC_inflow(ii_basin)
               if (AC_inflow_ciijj.gt.AC_max) then
                  AC_max=AC_inflow_ciijj
                  quota_max=quota_ciijj
                  z1=quota_ciijj
                  if (mod(p_outflow_ciijj,2).eq.1) then
                     d1=delta_xy
                  else ! if (mod(p_outflow_ciijj,2).eq.0) then
                     d1=delta_x
                  end if
               else if (AC_inflow_ciijj.eq.AC_max) then
                  AC_max=AC_inflow_ciijj ! non serve
                  if (quota_ciijj.gt.quota_max) then
                     quota_max=quota_ciijj
                     z1=quota_ciijj
                     if (mod(p_outflow_ciijj,2).eq.1) then
                        d1=delta_xy
                     else ! if (mod(p_outflow_ciijj,2).eq.0) then
                        d1=delta_x
                     end if
                  end if
               end if
             end if
         end do
      end do
      z2=quota
      z3=quota-local_slope_t*delta_x ! phantom
      d2=d1 ! phantom
      epl=d2
      local_slope=(z2-z3)/d2
      DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
      ASk=A_outflow*local_slope**kas
   end if

   if (local_slope.le.0.0E0) then
      write(6,*)'i_basin=',i_basin
      write(6,*)'local_slope=<0'
      local_slope=local_slope_t
   end if

   call set_epl(i_basin,epl)
   call set_local_slope(i_basin,local_slope)
   call set_ASk(i_basin,ASk)
   call set_DN(i_basin,DL)

   link_slope=local_slope
   if (link_slope.le.0.0E0) then
      write(6,*)'link_slope=',link_slope
      write(6,*)'i_basin=',i_basin
      stop "stop in lpf"
   end if

! write link_slope in the uppermost cells of the outlet link (this 
! will reported in the subsequent cell of the link by the subroutine HC)

   i_basin=i_basin_sorgente_l_link
   call set_link_slope(i_basin,  link_slope )

else ! (i_basin.ne.i_basin_chiusura) then

   iii=nint((p_outflow-5)/3.)
   jjj=p_outflow-5-3*iii
   i_basin_cv=i_basin+(M*iii+jjj)
   quota_cv=dtm_quota(i_basin_cv)
   link_number_cv=dtm_link_number(i_basin_cv) 

! calculate: (1) the elemenatal length of the channel within each DEM cell 
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

   if (link_number.eq.1) then
      if (mod(p_outflow,2).eq.1) then
         d2=delta_xy
      else ! if (mod(p_outflow,2).eq.0) then
         d2=delta_x
      end if
      d1=d2 ! phantom
      z1=quota ! phantom
      z2=quota
      z3=quota_cv
      epl=d2
      local_slope=(z2-z3)/d2
      DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
   else ! if (link_number.gt.1) then
      jr=mod(i_basin,M)
      if (jr.ne.0) then
         j=jr
         i=(i_basin-j)/M+1
      else
         j=M
         i=i_basin/M
      end if
      AC_max=0
      quota_max=-1000.0
      do ii=i-1,i+1
         do jj=j-1,j+1
            if (ii.eq.0 .or. ii.eq.N+1) cycle
            if (jj.eq.0 .or. jj.eq.M+1) cycle
            ii_basin=(ii-1)*M+jj
            if (dtm_index_pr(ii_basin) == 0) cycle
            p_outflow_ciijj= dtm_p_outflow(ii_basin)
            p_inflow=3*(ii-i)+(jj-j)+5
            if (p_inflow+p_outflow_ciijj.eq.10) then
               quota_ciijj=dtm_quota(ii_basin)
               AC_inflow_ciijj= dtm_AC_inflow(ii_basin)
               if (AC_inflow_ciijj.gt.AC_max) then     
                  AC_max=AC_inflow_ciijj
                  quota_max=quota_ciijj
                  z1=quota_ciijj
                  if (mod(p_outflow_ciijj,2).eq.1) then
                     d1=delta_xy
                  else ! if (mod(p_outflow_ciijj,2).eq.0) then
                     d1=delta_x
                  end if
               else if (AC_inflow_ciijj.eq.AC_max) then
                  AC_max=AC_inflow_ciijj ! non serve
                  if (quota_ciijj.gt.quota_max) then
                     quota_max=quota_ciijj
                     z1=quota_ciijj
                     if (mod(p_outflow_ciijj,2).eq.1) then
                        d1=delta_xy
                     else ! if (mod(p_outflow_ciijj,2).eq.0) then
                        d1=delta_x
                     end if
                  end if
               end if
            end if
         end do
      end do
      z2=quota
      z3=quota_cv
      if (mod(p_outflow,2).eq.1) then
         d2=delta_xy
      else ! if (mod(p_outflow,2).eq.0) then
         d2=delta_x
      end if
      epl=d2
      local_slope=(z2-z3)/d2
      DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
   end if
   ASk=A_outflow*local_slope**kas

   if (local_slope.le.0.0E0) then
      write(6,*)'i_basin=',i_basin
      write(6,*)'local_slope=<0'
      local_slope=local_slope_t
   end if

   call set_epl(i_basin,epl)
   call set_local_slope(i_basin,local_slope)
   call set_ASk(i_basin,ASk)
   call set_DN(i_basin,DL)

   link_slope_num=local_slope*epl
   link_slope_den=epl

! subsequent cells of the outlet link (the outlet cell should have logic
! numeber link_number=0 except it is the first cell of the link)

   link_number=link_number_cv
   do while(link_number.eq.0)
      i_basin_cm=i_basin
      quota_cm=dtm_quota(i_basin_cm)
      p_outflow_cm=dtm_p_outflow(i_basin_cm)
      i_basin=i_basin_cv
      quota=dtm_quota(i_basin)
      p_outflow=dtm_p_outflow(i_basin)
      link_number=dtm_link_number(i_basin)
      AC_inflow=dtm_AC_inflow(i_basin)
      A_outflow=AC_inflow*A_cell+A_cell

      if (i_basin.eq.i_basin_chiusura) then
         write(6,*) 'i_basin_chiusura = ',i_basin_chiusura

! calculate: (1) the elemenatal length of the channel within each DEM cell 
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

         if (mod(p_outflow_cm,2).eq.1) then
            d1=delta_xy
         else ! if (mod(p_outflow_cm,2).eq.0) then
            d1=delta_x
         end if
         z1=quota_cm
         z2=quota
         z3=quota-local_slope_t*delta_x ! phantom
         d2=d1 ! phantom
         epl=d2
         local_slope=(z2-z3)/d2
         DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
         ASk=A_outflow*local_slope**kas

         if (local_slope.le.0.0E0) then
            write(6,*)'i_basin=',i_basin
            write(6,*)'local_slope=<0'
            local_slope=local_slope_t
         end if

         call set_epl(i_basin,epl)
         call set_local_slope(i_basin,local_slope)
         call set_ASk(i_basin,ASk)
         call set_DN(i_basin,DL)

         link_slope_num=link_slope_num+local_slope*epl
         link_slope_den=link_slope_den+epl
         link_slope=link_slope_num/link_slope_den

         if (link_slope.le.0.0E0) then
            write(6,*)'link_slope=',link_slope
            write(6,*)'i_basin=',i_basin
            stop "stop in lpf"
         end if

! write link_slope in the uppermost cells of the outlet link (this 
! will reported in the subsequent cell of the link by the subroutine HC)

         i_basin=i_basin_sorgente_l_link
         call set_link_slope(i_basin,link_slope)

         link_number=1 ! ovvero esci dal ciclo do while

      else

         iii=nint((p_outflow-5)/3.)
         jjj=p_outflow-5-3*iii
         i_basin_cv=i_basin+(M*iii+jjj)
         quota_cv=dtm_quota(i_basin_cv)
         link_number_cv=dtm_link_number(i_basin_cv)

! calculate: (1) the elemenatal length of the channel within each DEM cell
! epl (it is assumend here that delta_x=delta_y); (2) the local slope
! (positive downward) local_slope; (3) the local convexity, DL; (4) the
! funcion ASk=AS**k, where k=kas

         z1=quota_cm
         z2=quota
         z3=quota_cv
         if (mod(p_outflow_cm,2).eq.1) then
            d1=delta_xy
         else ! if (mod(p_outflow_cm,2).eq.0) then
            d1=delta_x
         end if
         if (mod(p_outflow,2).eq.1) then
            d2=delta_xy
         else ! if (mod(p_outflow,2).eq.0) then
            d2=delta_x
         end if
         epl=d2
         local_slope=(z2-z3)/d2
         DL=((z3-z2)/d2-(z2-z1)/d1)/((d1+d2)/2) ! concavita'
         ASk=A_outflow*local_slope**kas

         if (local_slope.le.0.0E0) then
            write(6,*)'i_basin=',i_basin
            write(6,*)'local_slope=<0'
            local_slope=local_slope_t
         end if

         call set_epl(i_basin,epl)
         call set_local_slope(i_basin,local_slope)
         call set_ASk(i_basin,ASk)
         call set_DN(i_basin,DL)

         link_slope_num=link_slope_num+local_slope*epl
         link_slope_den=link_slope_den+epl
         link_number=link_number_cv ! ovvero esegui un altro giro nel ciclo 
      end if
   end do
end if
link_slope=link_slope_num/link_slope_den
if (link_slope.le.0.0E0) then
   write(6,*)'link_slope=',link_slope
   write(6,*)'i_basin=',i_basin
   stop "stop in lpf"
end if

! write link_slope in the uppermost cells of the outlet link (this
! will reported in the subsequent cell of the link by the subroutine HC)

i_basin=i_basin_sorgente_l_link
call set_link_slope(i_basin,link_slope)

! close the file ioi

close(17)

return
end subroutine lpf
