!-----------------------------------------------------------------------------
!
! MBBIO
!
! The module MBBIO manages the I/O between programs and binary files
! basin_b/basin_i.
!
!-----------------------------------------------------------------------------

module mbbio

use mpar

implicit none

private

public :: load_dtm,save_dtm,init_dtm,dtm_index_pr,set_dtm_index

public :: dtm_vh            !  1
public :: dtm_rl_min        !  2
public :: dtm_LAI           !  3
public :: dtm_vsc           !  4
public :: dtm_theta_s       !  5
public :: dtm_eta           !  6
public :: dtm_psi_s         !  7
public :: dtm_K_s           !  8
public :: dtm_Z_low         !  9
public :: dtm_Z_up          ! 10
public :: dtm_theta_r       ! 11
public :: dtm_rl_max        ! 12
public :: dtm_quota         ! 13
public :: dtm_p_outflow     ! 14
public :: dtm_cell_number   ! 15
public :: dtm_link_number   ! 16
public :: dtm_AC_inflow     ! 17
public :: dtm_kS_h          ! 18
public :: dtm_K_pp_h        ! 19
public :: dtm_epl           ! 20
public :: dtm_local_slope   ! 21
public :: dtm_link_slope    ! 22
public :: dtm_kS_c          ! 23
public :: dtm_K_pp_c        ! 24
public :: dtm_Wsn           ! 25
public :: dtm_b1sn          ! 26
public :: dtm_Psn           ! 27
public :: dtm_d1sn          ! 28
public :: dtm_Wgwn          ! 29
public :: dtm_b1gwn         ! 30
public :: dtm_Pgwn          ! 31
public :: dtm_d1gwn         ! 32
public :: dtm_ASk           ! 33
public :: dtm_DN            ! 34
public :: dtm_hcID          ! 35
public :: dtm_sumdev        ! 36

public :: set_vh            !  1
public :: set_rl_min        !  2
public :: set_LAI           !  3
public :: set_vsc           !  4
public :: set_theta_s       !  5
public :: set_eta           !  6
public :: set_psi_s         !  7
public :: set_K_s           !  8
public :: set_Z_low         !  9
public :: set_Z_up          ! 10
public :: set_theta_r       ! 11
public :: set_rl_max        ! 12
public :: set_quota         ! 13
public :: set_p_outflow     ! 14
public :: set_cell_number   ! 15
public :: set_link_number   ! 16
public :: set_AC_inflow     ! 17
public :: set_kS_h          ! 18
public :: set_K_pp_h        ! 19
public :: set_epl           ! 20
public :: set_local_slope   ! 21
public :: set_link_slope    ! 22
public :: set_kS_c          ! 23
public :: set_K_pp_c        ! 24
public :: set_Wsn           ! 25
public :: set_b1sn          ! 26
public :: set_Psn           ! 27
public :: set_d1sn          ! 28
public :: set_Wgwn          ! 29
public :: set_b1gwn         ! 30
public :: set_Pgwn          ! 31
public :: set_d1gwn         ! 32
public :: set_ASk           ! 33
public :: set_DN            ! 34
public :: set_hcID          ! 35
public :: set_sumdev        ! 36

integer,parameter :: nbb=36

type cella
   real ::    vh            !  1: vegetation height
   real ::    rl_min        !  2: minimum leaf resistance
   real ::    LAI           !  3: leaf area index
   real ::    vsc           !  4: vegetation storage capacity
   real ::    theta_s       !  5: saturation soil moisture content
   real ::    eta           !  6: pore-size distribution index
   real ::    psi_s         !  7: saturated soil matrix potential
   real ::    K_s           !  8: saturated soil conductivity
   real ::    Z_low         !  9: lower soil layer depth
   real ::    Z_up          ! 10: upper soil layer depth
   real ::    theta_r       ! 11: residual soil moisture content
   real ::    rl_max        ! 12: maximum leaf resistance
   real ::    quota         ! 13: cell elevation
   integer :: p_outflow     ! 14: pointer to the outflow cell
   integer :: cell_number   ! 15: cell number
   integer :: link_number   ! 16: link number
   integer :: AC_inflow     ! 17: upstream drainage area in cells
   real ::    kS_h          ! 18: hillslope Gauckler-Strickler coefficient
   real ::    K_pp_h        ! 19: hillslope preferential paths conductivity
   real ::    epl           ! 20: elemental path length
   real ::    local_slope   ! 21: local slope
   real ::    link_slope    ! 22: link slope
   real ::    kS_c          ! 23: channel Gauckler-Strickler coefficient
   real ::    K_pp_c        ! 24: channel preferential paths conductivity
   real ::    Wsn           ! 25: hydraulic gemetry parameter
   real ::    b1sn          ! 26: hydraulic gemetry parameter
   real ::    Psn           ! 27: hydraulic gemetry parameter
   real ::    d1sn          ! 28: hydraulic gemetry parameter
   real ::    Wgwn          ! 29: hydraulic gemetry parameter
   real ::    b1gwn         ! 30: hydraulic gemetry parameter
   real ::    Pgwn          ! 31: hydraulic gemetry parameter
   real ::    d1gwn         ! 32: hydraulic gemetry parameter
   real ::    ASk           ! 33: montgomery and dietrich function
   real ::    DN            ! 34: normalized divergence
   integer :: hcID          ! 35: hillslope/channel identifier
   real ::    sumdev        ! 36: cumulative deviation
end type

type(cella),dimension(:),allocatable :: dtm
integer,dimension(:),allocatable :: indice

type(cella),parameter :: cella_iniziale=cella(0.0, & ! vh
                                              0.0, & ! rl_min
                                              0.0, & ! LAI
                                              0.0, & ! vsc
                                              0.0, & ! theta_s
                                              0.0, & ! eta
                                              0.0, & ! psi_s
                                              0.0, & ! K_s
                                              0.0, & ! Z_low
                                              0.0, & ! Z_up
                                              0.0, & ! theta_r
                                              0.0, & ! rl_max
                                              0.0, & ! quota
                                              0  , & ! p_outflow
                                              0  , & ! cell_number
                                              0  , & ! link_number
                                              0  , & ! AC_inflow
                                              0.0, & ! kS_h
                                              0.0, & ! K_pp_h
                                              0.0, & ! epl
                                              0.0, & ! local_slope
                                              0.0, & ! link_slope
                                              0.0, & ! kS_c
                                              0.0, & ! K_pp_c
                                              0.0, & ! Wsn
                                              0.0, & ! b1sn
                                              0.0, & ! Psn
                                              0.0, & ! d1sn
                                              0.0, & ! Wgwn
                                              0.0, & ! b1gwn
                                              0.0, & ! Pgwn
                                              0.0, & ! d1gwn
                                              0.0, & ! ASk
                                              0.0, & ! DN
                                              0  , & ! hcID
                                              0.0)   ! sumdev

contains

!----------------------------------------------------------------------------- 
!
! INIT_DTM
!
! The subroutine INIT_DTM initializes the dtm data in the variable indice.
!  
!-----------------------------------------------------------------------------

subroutine init_dtm(n_rec)
integer,intent(in) :: n_rec

if (.not. allocated(indice)) then
   allocate(indice(M*N))
else
   write(6,*) "can't allocate indice"
   stop
end if
indice=0
if (.not. allocated(dtm)) then
   allocate(dtm(n_rec))
else
   write(6,*) "can't allocate dtm"
   stop
end if
dtm=cella_iniziale
end subroutine init_dtm

!----------------------------------------------------------------------------- 
!
! LOAD_DTM
!
! The subroutine LOAD_DTM loads the dtm data from the binary files 
! basin_b/basin_i.
!  
!-----------------------------------------------------------------------------

subroutine load_dtm(NomeDemFile,NomeIndice)
character(len=*),intent(in) :: NomeDemFile,NomeIndice
integer :: n_rec,i
open(22,file=NomeDemFile,access='direct',status='old', &
form='unformatted',recl=4*nbb)
open(23,file=NomeIndice,access='direct',status='old', &
form='unformatted',recl=4)
read(23,rec=1) n_rec
if (.not. allocated(indice)) then
   allocate(indice(M*N))
else
   write(6,*) "allocated(indice)",ubound(indice,1)
   stop
endif
if (.not. allocated(dtm)) then
   allocate(dtm(n_rec))
else
   write(6,*) "allocated(dtm)",ubound(dtm,1)
   stop
endif
do i=1, M*N
   read(23,rec=i+1) indice(i)
end do
do i=1, n_rec
    read(22,rec= i) dtm(i)
end do
close(22)
close(23)
end subroutine load_dtm

!----------------------------------------------------------------------------- 
!
! SAVE_DTM
!
! The subroutine SAVE_DTM saves the dtm data in the binary files 
! basin_b/basin_i.
!  
!-----------------------------------------------------------------------------

subroutine save_dtm(NomeDemFile,NomeIndice)
character(len=*),intent(in) :: NomeDemFile
character(len=*),intent(in),optional :: NomeIndice
integer :: i
open(22,file=NomeDemFile,access='direct',status='unknown', &
form='unformatted',recl=4*nbb)
do i=1,ubound(dtm,1)
   write(22,rec=i) dtm(i)
end do
close(22)
if (present(NomeIndice)) then
   open(23,file=NomeIndice,access='direct',status='unknown', &
   form='unformatted',recl=4)
   write(23,rec=1) ubound(dtm,1)
   do i=1,M*N
      write(23,rec=i+1) indice(i)
   end do
   close(23)
end if
deallocate(indice)
deallocate(dtm)
end subroutine save_dtm

!-----------------------------------------------------------------------------
!
! DTM_INDEX_PR
!
! The function DTM_INDEX_PR provides the index of records in basin_b 
! for parameter read. 
!
!-----------------------------------------------------------------------------

function dtm_index_pr(i)
integer :: dtm_index_pr
integer,intent(in) :: i
if (i <= 0 .or. i > ubound(indice,1)) stop &
"error in dtm_index_pr: input out of range!"
dtm_index_pr=indice(i)
end function dtm_index_pr

!-----------------------------------------------------------------------------
!
! DTM_INDEX_PS
!
! The function DTM_INDEX_PS provides the index of records in basin_b 
! for parameter set. 
!
!-----------------------------------------------------------------------------

function dtm_index_ps(i)
integer :: dtm_index_ps
integer,intent(in) :: i
if (i <= 0 .or. i > ubound(indice,1)) stop &
"error in dtm_index_ps: input out of range"
if (indice(i) <= 0) stop & 
"error in dtm_index_ps: cell write not allowed"
dtm_index_ps=indice(i)
end function dtm_index_ps

!-----------------------------------------------------------------------------
!
! SET_DTM_INDEX
!
! The function SET_DTM_INDEX sets the index of records in basin_b. 
!
!-----------------------------------------------------------------------------

subroutine set_dtm_index(i_basin,i_rec)
integer,intent(in) :: i_basin,i_rec
if (i_basin <= 0 .or. i_basin > ubound(indice,1) ) stop &
"error in set_dtm_index: input out of range"
indice(i_basin)=i_rec
end subroutine set_dtm_index

!-----------------------------------------------------------------------------
!
! Subroutines for setting parameter values.
!
!-----------------------------------------------------------------------------

subroutine set_vh(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%vh=x
end subroutine

subroutine set_rl_min(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%rl_min=x
end subroutine

subroutine set_LAI(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%LAI=x
end subroutine

subroutine set_vsc(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%vsc=x
end subroutine

subroutine set_theta_s(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%theta_s=x
end subroutine

subroutine set_eta(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%eta=x
end subroutine

subroutine set_psi_s(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%psi_s=x
end subroutine

subroutine set_K_s(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%K_s=x
end subroutine

subroutine set_Z_low(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Z_low=x
end subroutine

subroutine set_Z_up(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Z_up=x
end subroutine

subroutine set_theta_r(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%theta_r=x
end subroutine

subroutine set_rl_max(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%rl_max=x
end subroutine

subroutine set_quota(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%quota=x
end subroutine

subroutine set_p_outflow(i,j)
integer,intent(in) :: i
integer,intent(in) :: j
dtm(dtm_index_ps(i))%p_outflow=j
end subroutine

subroutine set_cell_number(i,j)
integer,intent(in) :: i
integer,intent(in) :: j
dtm(dtm_index_ps(i))%cell_number=j
end subroutine

subroutine set_link_number(i,j)
integer,intent(in) :: i
integer,intent(in) :: j
dtm(dtm_index_ps(i))%link_number=j
end subroutine

subroutine set_AC_inflow(i,j)
integer,intent(in) :: i
integer,intent(in) :: j
dtm(dtm_index_ps(i))%AC_inflow=j
end subroutine

subroutine set_kS_h(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%kS_h=x
end subroutine

subroutine set_K_pp_h(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%K_pp_h=x
end subroutine

subroutine set_epl(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%epl=x
end subroutine

subroutine set_local_slope(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%local_slope=x
end subroutine

subroutine set_link_slope(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%link_slope=x
end subroutine

subroutine set_kS_c(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%kS_c=x
end subroutine

subroutine set_K_pp_c(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%K_pp_c=x
end subroutine

subroutine set_Wsn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Wsn=x
end subroutine

subroutine set_b1sn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%b1sn=x
end subroutine

subroutine set_Psn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Psn=x
end subroutine

subroutine set_d1sn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%d1sn=x
end subroutine

subroutine set_Wgwn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Wgwn=x
end subroutine

subroutine set_b1gwn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%b1gwn=x
end subroutine

subroutine set_Pgwn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%Pgwn=x
end subroutine

subroutine set_d1gwn(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%d1gwn=x
end subroutine

subroutine set_ASk(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%ASk=x
end subroutine

subroutine set_DN(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%DN=x
end subroutine

subroutine set_hcID(i,j)
integer,intent(in) :: i
integer,intent(in) :: j
dtm(dtm_index_ps(i))%hcID=j
end subroutine

subroutine set_sumdev(i,x)
integer,intent(in) :: i
real,intent(in) :: x
dtm(dtm_index_ps(i))%sumdev=x
end subroutine

!-----------------------------------------------------------------------------
!  
! Functions for reading parameter values.
!
!-----------------------------------------------------------------------------

function dtm_vh(i)
real :: dtm_vh
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_vh=dtm(k)%vh
else
   stop "h"
end if
end function

function dtm_rl_min(i)
real :: dtm_rl_min
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_rl_min=dtm(k)%rl_min
else
   stop "rl_min"
end if
end function

function dtm_LAI(i)
real :: dtm_LAI
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_LAI=dtm(k)%LAI
else
   stop "rl_min"
end if
end function

function dtm_vsc(i)
real :: dtm_vsc
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_vsc=dtm(k)%vsc
else
   stop "vsc"
end if
end function

function dtm_theta_s(i)
real :: dtm_theta_s
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_theta_s=dtm(k)%theta_s
else
   stop "theta_s"
end if
end function

function dtm_eta(i)
real :: dtm_eta
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_eta=dtm(k)%eta
else
   stop "eta"
end if
end function

function dtm_psi_s(i)
real :: dtm_psi_s
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_psi_s=dtm(k)%psi_s
else
   stop "psi_s"
end if
end function

function dtm_K_s(i)
real :: dtm_K_s
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_K_s=dtm(k)%K_s
else
   stop "K_s"
end if
end function

function dtm_Z_low(i)
real :: dtm_Z_low
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_Z_low=dtm(k)%Z_low
else
   stop "Z_low"
end if
end function

function dtm_Z_up(i)
real :: dtm_Z_up
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_Z_up=dtm(k)%Z_up
else
   stop "Z_low"
end if
end function

function dtm_theta_r(i)
real :: dtm_theta_r
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_theta_r=dtm(k)%theta_r
else
   stop "theta_r"
end if
end function

function dtm_rl_max(i)
real :: dtm_rl_max
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_rl_max=dtm(k)%rl_max
else
   stop "rl_max"
end if
end function

function dtm_quota(i)
real :: dtm_quota
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_quota=dtm(k)%quota
else
   dtm_quota=-1.0
end if
end function

function dtm_p_outflow(i)
integer :: dtm_p_outflow
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_p_outflow=dtm(k)%p_outflow
else
   stop "dtm_p_outflow"
end if
end function

function dtm_cell_number(i)
integer :: dtm_cell_number
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_cell_number=dtm(k)%cell_number
else
   stop "dtm_cell_number"
end if
end function

function dtm_link_number(i)
integer :: dtm_link_number
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_link_number= dtm(k)%link_number
else
   stop "dtm_link_number"
end if
end function

function dtm_AC_inflow(i)
integer :: dtm_AC_inflow
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_AC_inflow=dtm(k)%AC_inflow
else
   stop "dtm_AC_inflow"
end if
end function

function dtm_kS_h(i)
real :: dtm_kS_h
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_kS_h=dtm(k)%kS_h
else
   stop "dtm_kS_h"
end if
end function

function dtm_K_pp_h(i)
real :: dtm_K_pp_h
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_K_pp_h=dtm(k)%K_pp_h
else
   stop "dtm_K_pp_h"
end if
end function

function dtm_epl(i)
real :: dtm_epl
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_epl=dtm(k)%epl
else
   stop "epl"
end if
end function

function dtm_local_slope(i)
real :: dtm_local_slope
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then
   dtm_local_slope=dtm(k)%local_slope
else
   stop "dtm_local_slope"
end if
end function

function dtm_link_slope(i)
real :: dtm_link_slope
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_link_slope=dtm(k)%link_slope
else
   stop "dtm_link_slope"
end if
end function

function dtm_kS_c(i)
real :: dtm_kS_c
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_kS_c=dtm(k)%kS_c
else
   stop "dtm_kS_c"
end if
end function

function dtm_K_pp_c(i)
real :: dtm_K_pp_c
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_K_pp_c=dtm(k)%K_pp_c
else
   stop "dtm_K_pp_c"
end if
end function

function dtm_Wsn(i)
real :: dtm_Wsn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_Wsn=dtm(k)%Wsn
else
   stop "dtm_Wsn"
end if
end function

function dtm_b1sn(i)
real :: dtm_b1sn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_b1sn=dtm(k)%b1sn
else
   stop "dtm_b1sn"
end if
end function

function dtm_Psn(i)
real :: dtm_Psn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_Psn= dtm(k)%Psn
else
   stop "dtm_Psn"
end if
end function

function dtm_d1sn(i)
real :: dtm_d1sn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_d1sn= dtm(k)%d1sn
else
   stop "dtm_d1sn"
end if
end function

function dtm_Wgwn(i)
real :: dtm_Wgwn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_Wgwn=dtm(k)%Wgwn
else
   stop "dtm_Wgwn"
end if
end function

function dtm_b1gwn(i)
real :: dtm_b1gwn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_b1gwn= dtm(k)%b1gwn
else
   stop "dtm_b1gwn"
end if
end function

function dtm_Pgwn(i)
real :: dtm_Pgwn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_Pgwn=dtm(k)%Pgwn
else
   stop "dtm_Pgwn"
end if
end function

function dtm_d1gwn(i)
real :: dtm_d1gwn
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_d1gwn= dtm(k)%d1gwn
else
   stop "dtm_d1gwn"
end if
end function

function dtm_ASk(i)
real :: dtm_ASk
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_ASk= dtm(k)%ASk
else
   stop "dtm_ASk"
end if
end function

function dtm_DN(i)
real :: dtm_DN
integer,intent(in):: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_DN= dtm(k)%DN
else
   stop "dtm_DN"
end if
end function

function dtm_hcID(i)
integer :: dtm_hcID
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_hcID= dtm(k)%hcID
else
   stop "dtm_hcID "
end if
end function

function dtm_sumdev(i)
real :: dtm_sumdev
integer,intent(in) :: i
integer :: k
k=dtm_index_pr(i)
if (k /= 0) then  
   dtm_sumdev=dtm(k)%sumdev
else
   stop "dtm_sumdev"
end if
end function

end module mbbio
