!-----------------------------------------------------------------------------
!
! MPAR
!
! Module containing definitions of parameters assigned in the parameter 
! file <parfile>. This module also contains three subroutines:
!
! 1) RPARFILE
!
! This subroutine reads <parfile> recognizing parameter descriptions 
! (before the "=" mark) and free-format parameters (after the "=" mark).
!
! 2) RROW
!
! This subroutine reads and analyze each row of <parfile>.
!
! 3) WPARFILE
!
! This subroutine rewrites <parfile> with specified changes.
!
!-----------------------------------------------------------------------------
 
module mpar
 
implicit none
 
integer(kind=4),private,parameter :: uparfile=12
real(kind=4) :: delta_x,delta_y,delta_t
integer(kind=4) :: N_celle,N,M,k_isim,k_fsim,k_db
 
real(kind=4) :: W_R0_sn_h,W_R0_sn_c,P_R0_sn_h,P_R0_sn_c
real(kind=4) :: b1_sn_h,b1_sn_c,d1_sn_h,d1_sn_c
real(kind=4) :: b2_sn_h,b2_sn_c,d2_sn_h,d2_sn_c
real(kind=4) :: W_R0_gwn_h,W_R0_gwn_c,P_R0_gwn_h,P_R0_gwn_c
real(kind=4) :: b1_gwn_h,b1_gwn_c,d1_gwn_h,d1_gwn_c
real(kind=4) :: b2_gwn_h,b2_gwn_c,d2_gwn_h,d2_gwn_c
real(kind=4) :: pt
real(kind=4) :: local_slope_t
integer(kind=4) :: imethod
real(kind=4) :: lambda
integer(kind=4) :: nchc
real(kind=4) :: A_threshold
real(kind=4) :: ASk_threshold
real(kind=4) :: DN_threshold
real(kind=4) :: kas
integer(kind=4) :: nr_sf
integer(kind=4) :: nr_gwf
real(kind=4) :: rk_Storage
real(kind=4) :: rk_theta_up
real(kind=4) :: rk_theta_low
real(kind=4) :: vh_1,vh_2
real(kind=4) :: rl_min_1,rl_min_2
real(kind=4) :: LAI_1,LAI_2
real(kind=4) :: vsc_1,vsc_2
real(kind=4) :: theta_s_1,theta_s_2
real(kind=4) :: eta_1,eta_2
real(kind=4) :: psi_s_1,psi_s_2
real(kind=4) :: K_s_1,K_s_2
real(kind=4) :: Z_low_1,Z_low_2
real(kind=4) :: Z_up_1,Z_up_2
real(kind=4) :: theta_r_1,theta_r_2
real(kind=4) :: rl_max_1,rl_max_2
real(kind=4) :: kS_h_1,kS_h_2
real(kind=4) :: K_pp_h_1,K_pp_h_2
real(kind=4) :: Q_sf_outlet_i
real(kind=4) :: Q_gwf_outlet_i
real(kind=4) :: kS_c_1,kS_c_2
real(kind=4) :: K_pp_c_1,K_pp_c_2

public :: rparfile,wparfile

contains

!-----------------------------------------------------------------------------
!
! RPARFILE
!
!-----------------------------------------------------------------------------

subroutine rparfile(parfile)

character(len=*),intent(in) :: parfile
character(len=512) :: row
integer(kind=4) :: ic
 
open(uparfile,file=parfile,form="formatted",action="read")

! read all the lines of <parfile>

! structural parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) delta_x
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) delta_y
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) N
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) M
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) N_celle
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) delta_t
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) k_db
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) k_isim
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) k_fsim
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) nr_sf
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) nr_gwf
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! terrain analysis parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) pt
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) local_slope_t
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) imethod
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) lambda
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) nchc
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) A_threshold
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) ASk_threshold
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) DN_threshold
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) kas
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! soil parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) theta_r_1,theta_r_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) theta_s_1,theta_s_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) eta_1,eta_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) psi_s_1,psi_s_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) K_s_1,K_s_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) Z_up_1,Z_up_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) Z_low_1,Z_low_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) rk_theta_up
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) rk_theta_low
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! vegetation parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) vh_1,vh_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) vsc_1,vsc_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) LAI_1,LAI_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) rl_min_1,rl_min_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) rl_max_1,rl_max_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) rk_Storage
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! atmosphere parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

! rivulet network parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) kS_h_1,kS_h_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! channel network parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) W_R0_sn_h,W_R0_sn_c,P_R0_sn_h,P_R0_sn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) b1_sn_h,b1_sn_c,d1_sn_h,d1_sn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) b2_sn_h,b2_sn_c,d2_sn_h,d2_sn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) W_R0_gwn_h,W_R0_gwn_c,P_R0_gwn_h,P_R0_gwn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) b1_gwn_h,b1_gwn_c,d1_gwn_h,d1_gwn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) b2_gwn_h,b2_gwn_c,d2_gwn_h,d2_gwn_c
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) kS_c_1,kS_c_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) Q_sf_outlet_i
if (ic/=0) then
   stop "error when reading the parameter file"
end if

! subsurface network parameters

read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"
read(uparfile,"(a)",iostat=ic) row ! read a comment line
if (ic/=0) stop "error when reading the parameter file"

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) K_pp_h_1,K_pp_h_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) K_pp_c_1,K_pp_c_2
if (ic/=0) then
   stop "error when reading the parameter file"
end if

call rrow(row,ic)
if (ic/=0) stop "error when reading the parameter file"
read(row,*,iostat=ic) Q_gwf_outlet_i
if (ic/=0) then
   stop "error when reading the parameter file"
end if

close(uparfile)

end subroutine rparfile

!-----------------------------------------------------------------------------
!
! RROW
!
!-----------------------------------------------------------------------------

subroutine rrow(row,ic)

character(len=*),intent(out) :: row
integer(kind=4),intent(out) :: ic
integer(kind=4) i

do
   read(uparfile,"(a)",iostat=ic) row
   if (ic/=0) return
   i=index(row,"=")
   if (i>0) exit
end do
row=row(i+1:)

end subroutine rrow

!-----------------------------------------------------------------------------
!
! WPARFILE
!
!-----------------------------------------------------------------------------

subroutine wparfile(parfile)

character(len=*),intent(in) :: parfile

open(uparfile,file=parfile,form="formatted")
rewind(uparfile)

! structural parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "STRUCTURAL PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,6x,f10.2)") &
"Grid spacing along the x-direction = ",delta_x

write(uparfile,"(a,6x,f10.2)") &
"Grid spacing along the y-direction = ",delta_y

write(uparfile,"(a,i7)") &
"DEM rectangle size along the x-direction = ",N

write(uparfile,"(a,i7)") &
"DEM rectangle size along the y-direction = ",M

write(uparfile,"(a,1x,i10)") &
"Number of cells within the catchment = ",N_celle

write(uparfile,"(a,24x,f10.1)") &
"Time resolution = ",delta_t

write(uparfile,"(a,8x,i10)") &
"Size of the simulation period = ",k_db

write(uparfile,"(a,i10)") &
"Time interval of the simulation start = ",k_isim

write(uparfile,"(a,2x,i10)") &
"Time interval of the simulation end = ",k_fsim

write(uparfile,"(a,5x,i10)") &
"Time interval subdivisions (SFP) = ",nr_sf

write(uparfile,"(a,4x,i10)") &
"Time interval subdivisions (SSFP) = ",nr_gwf

! terrain analysis parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "TERRAIN ANALYSIS PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,24x,1e10.3)") &
"Depit threshold slope = ",pt

write(uparfile,"(a,25x,1e10.3)") &
"Path threshold slope = ",local_slope_t

write(uparfile,"(a,3x,i4)") &
"Drainage directions method (LAD:1,LTD:2) = ",imethod

write(uparfile,"(a,3x,1f10.2)") &
"D8-LAD/D8-LTD method dampening factor = ",lambda

write(uparfile,"(a,1x,i4)") &
"Channel initiation method (A:1,ASk:2,ND:3) = ",nchc

write(uparfile,"(a,26x,1f10.2)") &
"Threshold area = ",A_threshold

write(uparfile,"(a,10x,1f10.2)") &
"Threshold for the ASk function = ",ASk_threshold

write(uparfile,"(a,6x,1e10.3)") &
"Threshold for the normalized divergence = ",DN_threshold

write(uparfile,"(a,10x,1f10.2)") &
"Exponent k of the ASk function = ",kas

! soil parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "SOIL PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,8x,2f10.3)") &
"Residual soil water content range = ",theta_r_1,theta_r_2

write(uparfile,"(a,6x,2f10.3)") &
"Saturation soil water content range = ",theta_s_1,theta_s_2

write(uparfile,"(a,7x,2f10.3)") &
"Pore-size distribution index range = ",eta_1,eta_2

write(uparfile,"(a,4x,2f10.2)") &
"Saturated soil water potential range = ",psi_s_1,psi_s_2

write(uparfile,"(a,6x,2e10.3)") &
"Saturated soil water conductivity range = ",K_s_1,K_s_2

write(uparfile,"(a,12x,2f10.2)") &
"Upper soil layer depth range = ",Z_up_1,Z_up_2

write(uparfile,"(a,12x,2f10.2)") &
"Lower soil layer depth range = ",Z_low_1,Z_low_2

write(uparfile,"(a,11x,1f10.3)") &
"Initial upper layer saturation = ",rk_theta_up

write(uparfile,"(a,11x,1f10.3)") &
"Initial lower layer saturation = ",rk_theta_low

! vegetation parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "VEGETATION PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,17x,2f10.2)") &
"Vegetation height range = ",vh_1,vh_2

write(uparfile,"(a,22x,2f10.4)") &
"Canopy storage range = ",vsc_1,vsc_2

write(uparfile,"(a,19x,2f10.2)") &
"Leaf Area Index range = ",LAI_1,LAI_2

write(uparfile,"(a,11x,2f10.2)") &
"Minimum leaf resistance range = ",rl_min_1,rl_min_2

write(uparfile,"(a,11x,2f10.2)") &
"Maximum leaf resistance range = ",rl_max_1,rl_max_2

write(uparfile,"(a,19x,1f10.3)") &
"Initial canopy wetness = ",rk_Storage

! atmosphere parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "ATMOSPHERE PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

! rivulet network parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "RIVULET NETWORK PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,10x,2f10.2)") &
"GS resistance coefficent range = ",kS_h_1,kS_h_2

! channel network parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "CHANNEL NETWORK PARAMETERS"
write(uparfile,"(a)") &
"------------------------------------------------------------------------------"

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",W_R0_sn_h,W_R0_sn_c,P_R0_sn_h,P_R0_sn_c

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",b1_sn_h,b1_sn_c,d1_sn_h,d1_sn_c

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",b2_sn_h,b2_sn_c,d2_sn_h,d2_sn_c

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",W_R0_gwn_h,W_R0_gwn_c,P_R0_gwn_h,P_R0_gwn_c

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",b1_gwn_h,b1_gwn_c,d1_gwn_h,d1_gwn_c

write(uparfile,"(a,14x,4f8.3)") &
"Hydraulic geometry parameters = ",b2_gwn_h,b2_gwn_c,d2_gwn_h,d2_gwn_c

write(uparfile,"(a,9x,2f10.2)") &
"GS resistance coefficient range = ",kS_c_1,kS_c_2

write(uparfile,"(a,1x,1f10.2)") &
"Initial surface discharge at the outlet = ",Q_sf_outlet_i

! subsurface network parameters

write(uparfile,"(a)") &
"------------------------------------------------------------------------------"
write(uparfile,"(a)") "SUBSURFACE NETWORK PARAMETERS"
write(uparfile,"(a)") & 
"------------------------------------------------------------------------------"

write(uparfile,"(a,12x,2e10.3)") &
"Hillslope soil conductivity range = ",K_pp_h_1,K_pp_h_2

write(uparfile,"(a,14x,2e10.3)") &
"Channel soil conductivity range = ",K_pp_c_1,K_pp_c_2

write(uparfile,"(a,1x,1f10.2)") &
"Initial surface discharge at the outlet = ",Q_gwf_outlet_i

close(uparfile)

end subroutine wparfile

end module mpar
