!-----------------------------------------------------------------------------
!
! RBB
!
! The program RBB reads the parameters contained in the binary files
! basin_b/basin_i and writes an DTM ascii file as output. This output
! file is called rdtm.dat or idtm.dat depending upon the parameter is
! real or integer, respectively.
!
!-----------------------------------------------------------------------------

program rbb

use mpar
use mbbio

implicit none

integer(kind=4) i,j,i_basin
integer :: ibb,ips,jd(0:9)

integer(kind=4) :: idtm_max_i,idtm_min_i,idtm_max,idtm_min
real(kind=4) :: rdtm_max_i,rdtm_min_i,rdtm_max,rdtm_min
integer(kind=4) :: sum_idtm,sum_c
real(kind=4) :: sum_rdtm,rmed_idtm,rmed_rdtm
real(kind=4) :: nodata
integer(kind=4) :: ic
character(len=32) :: dtmfile
character(len=24) :: fstr
character(len=24) :: nstr

real(kind=4),allocatable :: rdtm(:,:)
integer(kind=4),allocatable :: idtm(:,:)
integer,external :: parsel

! read <parfile>

call rparfile('hap.in')

allocate(rdtm(N,M))
allocate(idtm(N,M))

call load_dtm("basin_b","basin_i")

! select the output DTM file name

write(6,*)
write(6,*) 'Select the output DTM file name:'
write(6,*) '(Ctrl C or Enter to exit)'
write(6,*)
write(6,*) '-> '
read(5,'(a)') dtmfile
if (dtmfile == " ") stop
open(80,file=dtmfile,status='unknown',iostat=ic)
do while (ic /= 0)
   write(6,*) 'Invalid selection: try again...'
   write(6,*) '(Ctrl C or Enter to exit)'
   write(6,*) 
   write(6,*) '-> '
   read(5,'(a)') dtmfile
   if (dtmfile == " ") stop
   open(80,file=dtmfile,status='unknown',iostat=ic)
end do

! select the nodata threshold

write(6,*)
write(6,*) 'Select the notada value:'
write(6,*) '(Ctrl C to exit)'
write(6,*)
write(6,*) '-> '
read(5,*,iostat=ic) nodata
do while (ic /= 0)
   write(6,*) 'Invalid selection: try again...'
   write(6,*) '(Ctrl C to exit)'
   write(6,*)
   write(6,*) '-> '
   read(5,*,iostat=ic) nodata
end do

! select the parameter

ibb=parsel()

if (ibb == 14 ) then
!  two cases are possible:
!  1) HAP pointer system
!  2) Arc/Gis pointer system (Jenson and Domingue, 1988)
   write(6,*) 'Select the pointer system:'
   write(6,*) '1) HAP system'
   write(6,*) '2) Arc/Gis system'
   write(6,*) '(Ctrl C to exit)'
   write(6,*)
   write(6,*) '-> '
   read(5,*,iostat=ic) ips
   do while (ic /= 0)
      write(6,*) 'Invalid selection: try again...'
      write(6,*) '(Ctrl C to exit)'
      write(6,*)
      write(6,*) '-> '
      read(5,*,iostat=ic) ips
   end do
   jd(0)=0 ! outlet cell 
   jd(1)=8
   jd(2)=16
   jd(3)=32
   jd(4)=4
   jd(5)=0 ! this case should never verify
   jd(6)=64
   jd(7)=2
   jd(8)=1
   jd(9)=128
end if

! data creation

sum_c=0
idtm_max_i=0
idtm_min_i=1000000000
idtm_max=idtm_max_i
idtm_min=idtm_min_i
sum_idtm=0
rdtm_max_i=0.0E0
rdtm_min_i=1.0E30
rdtm_max=rdtm_max_i
rdtm_min=rdtm_min_i
sum_rdtm=0.0E0
do i=1,N
   do j=1,M
      i_basin=(i-1)*M+j
      if (dtm_index_pr(i_basin) == 0)  then
         select case (ibb)
         case (14:17,35)
            idtm(i,j)=int(nodata)
         case default
            rdtm(i,j)=nodata
         end select
      else
         sum_c=sum_c+1
         select case (ibb)
         case (14:17,35)
            select case (ibb)
            case (14)
               select case (ips)
               case (1)
                  idtm(i,j)=dtm_p_outflow(i_basin)
               case (2)
                  idtm(i,j)=jd(dtm_p_outflow(i_basin))
               case default
                  stop "unexpected case"
               end select
            case (15)
               idtm(i,j)=dtm_cell_number(i_basin)
            case (16)
               idtm(i,j)=dtm_link_number(i_basin)
            case (17)
               idtm(i,j)=dtm_AC_inflow(i_basin)
            case (35)
               idtm(i,j)=dtm_hcID(i_basin)
            case default
               stop "unexpected case"
            end select
            sum_idtm=sum_idtm+idtm(i,j)
            if (idtm(i,j).gt.idtm_max) idtm_max=idtm(i,j)
            if (idtm(i,j).lt.idtm_min) idtm_min=idtm(i,j)
         case default
            select case (ibb)
            case (1)
               rdtm(i,j)=dtm_vh(i_basin)
            case (2)
               rdtm(i,j)=dtm_rl_min(i_basin)
            case (3)
               rdtm(i,j)=dtm_LAI(i_basin)
            case (4)
               rdtm(i,j)=dtm_vsc(i_basin)
            case (5)
               rdtm(i,j)=dtm_theta_s(i_basin)
            case (6)
               rdtm(i,j)=dtm_eta(i_basin)
            case (7)
               rdtm(i,j)=dtm_psi_s(i_basin)
            case (8)
               rdtm(i,j)=dtm_K_s(i_basin)
            case (9)
               rdtm(i,j)=dtm_Z_low(i_basin)
            case (10)
               rdtm(i,j)=dtm_Z_up(i_basin)
            case (11)
               rdtm(i,j)=dtm_theta_r(i_basin)
            case (12)
               rdtm(i,j)=dtm_rl_max(i_basin)
            case (13)
               rdtm(i,j)=dtm_quota(i_basin)
            case (18)
               rdtm(i,j)=dtm_kS_h(i_basin)
            case (19)
               rdtm(i,j)=dtm_K_pp_h(i_basin)
            case (20)
               rdtm(i,j)=dtm_epl(i_basin)
            case (21)
               rdtm(i,j)=dtm_local_slope(i_basin)
            case (22)
               rdtm(i,j)=dtm_link_slope(i_basin)
            case (23)
               rdtm(i,j)=dtm_kS_c(i_basin)
            case (24)
               rdtm(i,j)=dtm_K_pp_c(i_basin)
            case (25)
               rdtm(i,j)=dtm_Wsn(i_basin)
            case (26)
               rdtm(i,j)=dtm_b1sn(i_basin)
            case (27)
               rdtm(i,j)=dtm_Psn(i_basin)
            case (28)
               rdtm(i,j)=dtm_d1sn(i_basin)
            case (29)
               rdtm(i,j)=dtm_Wgwn(i_basin)
            case (30)
               rdtm(i,j)=dtm_b1gwn(i_basin)
            case (31)
               rdtm(i,j)=dtm_Pgwn(i_basin)
            case (32)
               rdtm(i,j)=dtm_d1gwn(i_basin)
            case (33)
               rdtm(i,j)=dtm_ASk(i_basin)
            case (34)
               rdtm(i,j)=dtm_DN(i_basin)
            case (36)
               rdtm(i,j)=dtm_sumdev(i_basin)
            case default
               stop "unexpected case"
            end select
            sum_rdtm=sum_rdtm+rdtm(i,j)
            if (rdtm(i,j).gt.rdtm_max) rdtm_max=rdtm(i,j)
            if (rdtm(i,j).lt.rdtm_min) rdtm_min=rdtm(i,j)
         end select
      end if
   end do
end do
write(6,*) "number of cells = ",sum_c

! integer parameter?

if (idtm_max >= idtm_min) then
   write(6,*)'max value=',idtm_max
   write(6,*)'min value=',idtm_min
   if (idtm_max.eq.idtm_max_i) then
      write(6,*) 'initial value unchanged'
      stop
   end if
   if (idtm_min.eq.idtm_min_i) then
      write(6,*) 'initial value unchanged'
      stop
   end if
   rmed_idtm=real(sum_idtm)/sum_c
   write(6,*)'number of cells=',sum_c
   write(6,*)'mean value=',rmed_idtm

! write a integer output file

   write(fstr,'(i10)') N
   idtm_min=min(idtm_min,int(nodata))  
   idtm_max=max(idtm_max,abs(int(nodata)))  
   if (idtm_min.lt.0) then
      write(nstr,'(i10)') int(log10(real(idtm_max)))+3
   else 
      write(nstr,'(i10)') int(log10(real(idtm_max)))+2
   end if
   fstr='('//trim(adjustl(fstr))//'i'//trim(adjustl(nstr))//')'
   do j=M,1,-1
      write(80,fstr) (idtm(i,j),i=1,N)
   end do
end if

! real parameter?

if (rdtm_max >= rdtm_min) then
   write(6,*)'max value=',rdtm_max
   write(6,*)'min value=',rdtm_min
   if (abs(rdtm_max-rdtm_max_i).lt.1.0E-06) then
      write(6,*) 'initial value unchanged'
      stop
   end if
   if (abs(rdtm_min-rdtm_min_i).lt.1.0E-06) then
      write(6,*) 'initial value unchanged'
      stop
   end if
   rmed_rdtm=real(sum_rdtm)/sum_c
   write(6,*)'number of cells=',sum_c
   write(6,*)'mean value=',rmed_rdtm

! write a real output file

   write(fstr,'(i10)') N
   rdtm_min=min(rdtm_min,nodata)  
   if (rdtm_min.lt.0.0) then
      fstr='('//trim(adjustl(fstr))//'e14.6'//')'
   else
      fstr='('//trim(adjustl(fstr))//'e13.6'//')'
   end if
   do j=M,1,-1
      write(80,fstr) (rdtm(i,j),i=1,N)
   end do
end if

! close the file

close(80)

end program rbb

!-----------------------------------------------------------------------------
!
! PARSEL
!
! The function PARSEL perform the parameter selction task. 
!
!-----------------------------------------------------------------------------

function parsel()

implicit none

integer :: parsel
integer :: ibb
integer :: ic

call maschera()
read(5,*,iostat=ic) ibb
if (ibb <=0 .or. ibb > 36) ic=1
do while (ic /= 0) 
   write(6,*) 'Invalid selection: try again...'
   call maschera()
   read(5,*,iostat=ic) ibb
   if (ibb <=0 .or. ibb > 36) ic=1
end do
write(6,*)

parsel=ibb

contains

!-----------------------------------------------------------------------------
!
! MASCHERA
!
! The subroutine MASCHERA provides the mask in the parameter selction task. 
!
!-----------------------------------------------------------------------------

subroutine maschera()
write(6,*) 'Select the parameter to be stored:'
write(6,*) '(Ctrl C to exit)'
write(6,*)
write(6,*) ' 1) vh                    21) local_slope'
write(6,*) ' 2) rl_min                22) link_slope'
write(6,*) ' 3) LAI                   23) kS_c'
write(6,*) ' 4) vsc                   24) K_pp_c'
write(6,*) ' 5) theta_s               25) Wsn'
write(6,*) ' 6) eta                   26) b1sn'
write(6,*) ' 7) psi_s                 27) Psn'
write(6,*) ' 8) K_s                   28) d1sn'
write(6,*) ' 9) Z_low                 29) Wgwn'
write(6,*) '10) Z_up                  30) b1gwn'
write(6,*) '11) theta_r               31) Pgwn'
write(6,*) '12) rl_max                32) d1gwn'
write(6,*) '13) quota                 33) ASk'
write(6,*) '14) p_outflow             34) DN'
write(6,*) '15) cell_number           35) hcID'
write(6,*) '16) link_number           36) sumdev'          
write(6,*) '17) AC_inflow'
write(6,*) '18) kS_h'
write(6,*) '19) K_pp_h'
write(6,*) '20) epl'
write(6,*)
write(6,*) '-> '
end subroutine

end function
