!-----------------------------------------------------------------------------
!
! WBB
!
! This program defines the structure of the binary files basin_b/basin_i
! using the ascii DEM file <demfile>, which contains the catchment cell 
! elevations. The following reference system is considered:
!
!              ^
!              |
!              |
!    j=[1,M],y |
!              |
!              |
!             -|--------------->
!                  i=[1,N],x
!
!-----------------------------------------------------------------------------

program wbb

use mpar
use mbbio

implicit none

integer(kind=4) i,j,n_rec,i_basin, i_rec
real(kind=4) nodata
integer :: ic
character(len=32) demfile

real(kind=4),allocatable :: rwa(:)

integer,external :: catchment_cells

! set the DEM file name and nodata threshold
! (catchment cell are those with DEM elevation greater that nodata)

demfile='dtm_13.val'
nodata=0.0

! read <parfile>

call rparfile('hap.in')

allocate(rwa(N))

! open the ascii catchment DEM file <demfile>

open(73,file=demfile,status='old')
n_rec=catchment_cells(nodata,rwa)

call init_dtm(n_rec)

! write the binary files basin_b/basin_i

i_rec= 0
rewind 73
do j=M,1,-1
   read(73,*,iostat=ic) (rwa(i),i=1,N)
   select case(ic)
   case(:-1)
      write(6,*) "error when reading the file ",demfile
      stop
   case(0)
      do i=1,N
         if (rwa(i).gt.nodata) then
            i_rec=i_rec+1
            i_basin=(i-1)*M+j
            call set_dtm_index(i_basin,i_rec)
            call set_quota(i_basin,rwa(i))
         end if
      end do
   case(1:)
      write(6,*) "insufficient data in the file ",demfile
      stop
   end select
end do
WRITE(6,*) 'number of processed cells = ',n_rec

! close the files

close(73)

call save_dtm("basin_b","basin_i")

end program wbb


function catchment_cells(nodata, rwa)
use mpar
implicit none
integer :: catchment_cells
real,intent(in) :: nodata
real,dimension(n) :: rwa
integer :: n_rec,i,ic
rewind 73
n_rec= 0
do
   read(73,*,iostat=ic) (rwa(i),i=1,N)
   if (ic /= 0) exit
   do i=1,N
      if (rwa(i).gt.nodata) n_rec=n_rec+1
   end do
end do
catchment_cells=n_rec
rewind 73
end function catchment_cells
